@@ -10,7 +10,6 @@ from io import BytesIO
 try: from tinytag import TinyTag
 except ModuleNotFoundError: TinyTag = None
 
-
 class Client(object):
     def __init__(self, auth):
         self.__requests = Requests(auth)
@@ -20,6 +19,7 @@ class Client(object):
         self._auth = auth
         self.tools = Tools()
         self.__pool_manager = PoolManager()
+        self.run = asyncio.run
 
     async def sendMessage(self, object_guid, text, reply_to_message_id=None, meta_data=None):
         data = {'object_guid': object_guid, 'rnd': f'{randint(100000, 999999999)}', 'text': text, 'reply_to_message_id': reply_to_message_id}
@@ -32,18 +32,9 @@ class Client(object):
 
     async def sendPhoto(self, object_guid, image, caption=None, reply_to_message_id=None):
         file_name = f'shayan-heidari{randint(1, 999)}.jpg'
-        data = {
-            'file_inline': {
-                'dc_id': None,
-                'file_id': None,
-                'type':'Image',
-                'file_name': file_name,
-                'size': None,
-                'mime': 'jpg',
-                'access_hash_rec': is_uploaded,
-                'width': None,
-                'height': None,
-                'thumb_inline': None
+        data = {'file_inline': {'dc_id': None, 'file_id': None, 'type':'Image',
+                'file_name': file_name, 'size': None, 'mime': 'jpg', 'access_hash_rec': None,
+                'width': None, 'height': None, 'thumb_inline': None
             },
             'object_guid': object_guid, 'text': caption, 'rnd': f'{randint(100000,999999999)}', 'reply_to_message_id': reply_to_message_id
         }
@@ -69,14 +60,13 @@ class Client(object):
                 my_file.close()
             content_length = str(len(file))
             requestSendFile = await self.requestSendFile(file_name, 'jpg', content_length)
-            is_uploaded = await self.upload(requestSendFile.get('upload_url'), requestSendFile.get('access_hash_send'), requestSendFile.get('id'), file)
-            width, height = self.tools.getImageSize(file_bytes)
-            data['file_inline']['thumb_inline'] = self.tools.getThumbnail(file_bytes).decode('utf-8')
+            width, height = self.tools.getImageSize(file)
+            data['file_inline']['thumb_inline'] = self.tools.getThumbnail(file).decode('utf-8')
             data['file_inline']['size'] = content_length
             data['file_inline']['width'] = width
             data['file_inline']['height'] = height
             data['file_inline']['dc_id'] = requestSendFile.get('dc_id')
-            data['file_inline']['access_hash_rec'] = is_uploaded
+            data['file_inline']['access_hash_rec'] = await self.upload(requestSendFile.get('upload_url'), requestSendFile.get('access_hash_send'), requestSendFile.get('id'), file)
             data['file_inline']['file_id'] = requestSendFile.get('id')
 
         return await self.send('sendMessage', data, 5)
@@ -227,19 +217,6 @@ class Client(object):
 
         return await self.send('sendMessage', data, 5)
 
-    async def get_voice_duration(self, file_bytes):
-        file = BytesIO()
-        file.write(file_bytes)
-        file.seek(0)
-        audio = MP3(file)
-        return audio.info.length
-
-    async def getVideoDuration(self, video):
-        if TinyTag != None:
-            return round(TinyTag.get(video).duration * 1000)
-        else:
-            raise ImportWarning('Plaese install <TinyTag> and try again')
-
     async def sendGif(self, object_guid, gif, caption=None, reply_to_message_id=None):
         mime = f".{gif.split('.')[-1]}"
         file_name = f'shayan-heidari{randint(1, 999)}{mime}'
@@ -316,6 +293,19 @@ class Client(object):
 
         return await self.send('sendMessage', data, 5)
 
+    def get_voice_duration(self, file_bytes):
+        file = BytesIO()
+        file.write(file_bytes)
+        file.seek(0)
+        audio = MP3(file)
+        return audio.info.length
+
+    async def getVideoDuration(self, video):
+        if TinyTag != None:
+            return round(TinyTag.get(video).duration * 1000)
+        else:
+            raise ImportWarning('Plaese install <TinyTag> and try again')
+
     async def uploadAvatar(self, object_guid, image, thumbnail_file_id=None):
         data = {'object_guid': object_guid, 'thumbnail_file_id': None, 'main_file_id': None}
         with open(image, 'rb') as file:
@@ -435,7 +425,12 @@ class Client(object):
         return await self.send('forwardMessages', data, 5)
 
     async def seenChats(self, seen_list):
-        return await self.send('seenChats', {'seen_list': seen_list}, 5)
+        '''
+            The message ID must be of integer type
+            An Example:
+                {"object_guid": 341386188755760}
+        '''
+        return await self.send('seenChats', {'seen_list': seen_list})
 
     async def sendChatActivity(self, object_guid, action):
         data = {'activity': action, 'object_guid': object_guid}
@@ -498,8 +493,63 @@ class Client(object):
         data = {'object_guid': object_guid, 'avatar_id': avatar_id}
         return await self.send('deleteAvatar', data, 5)
 
-    async def download(self):
-        pass
+    async def download(self, object_guid=None, message_id=None, from_link=None, save=False, file_name=None, **kwargs):
+        file = b''
+
+        file_id = kwargs.get('file_id')
+        if file_id != None:
+            size = kwargs.get('size')
+            dc_id = kwargs.get('dc_id')
+            access_hash_rec = kwargs.get('access_hash_rec')
+        elif from_link != None and from_link.startswith('http'):
+            geted_data = await self.getLinkFromAppUrl(from_link)
+            geted_data = geted_data.get('link').get('open_chat_data')
+            geted_data = await self.getMessagesByID(geted_data.get('object_guid'), [geted_data.get('message_id')])
+            message_data = geted_data.get('data').get('messages')[0].get('file_inline')
+            file_id = message_data.get('file_id')
+            size = message_data.get('size')
+            dc_id = message_data.get('dc_id')
+            access_hash_rec = message_data.get('access_hash_rec')
+            file_name = message_data.get('file_name')
+        elif object_guid and message_id != None:
+            geted_data = await self.getMessagesByID(object_guid, [message_id])
+            message_data = geted_data.get('data').get('messages')[0].get('file_inline')
+            file_id = message_data.get('file_id')
+            size = message_data.get('size')
+            dc_id = message_data.get('dc_id')
+            access_hash_rec = message_data.get('access_hash_rec')
+            file_name = message_data.get('file_name')
+
+        url = 'https://messenger{}.iranlms.ir/GetFile.ashx'.format(dc_id)
+        headers = {
+            'auth': self._auth,
+            'file-id': file_id,
+			'access-hash-rec': access_hash_rec
+        }
+        start_index = 0
+
+        while True:
+            if start_index <= 131072:
+                headers['start-index'], headers['last-index'] = '0', str(size)
+                response = self.__pool_manager.request('GET', url, headers=headers)
+                if response.status == 200:
+                    file += response.data
+                    if save:
+                        with open(file_name, 'wb+') as my_file:
+                            my_file.write(file)
+                            my_file.close()
+                            return True
+                    else: return file
+            else:
+                for i in range(0, size, 131072):
+                    headers['start-index'], headers['last-index'] = str(i), str(i + 131072 if i + 131072 <= size else size)
+                    response = self.__pool_manager.request('GET', url, headers=headers)
+                    if response.status == 200:
+                        file += response.data
+                        continue
+
+    async def seenChannelMessages(self, channel_guid, min_id, max_id):
+        return await self.send('getChannelInfo', {"channel_guid": channel_guid, "max_id": int(max_id), "min_id": int(min_id)})
 
     async def getChannelInfo(self, channel_guid):
         return await self.send('getChannelInfo', {'channel_guid': channel_guid}, 5)
@@ -509,6 +559,15 @@ class Client(object):
 
     async def getChannelLink(self, channel_guid):
         return await self.send('getChannelLink', {'channel_guid': channel_guid}, 5)
+    
+    async def setChannelLink(self, channel_guid):
+        return await self.send('setChannelLink', {'channel_guid': channel_guid})
+
+    async def channelPreviewByJoinLink(self, channel_link):
+        return await self.send('channelPreviewByJoinLink', {'hash_link': channel_link.split('/')[-1]})
+
+    async def joinChannelByLink(self, channel_link):
+        return await self.send('channelPreviewByJoinLink', {'hash_link': channel_link.split('/')[-1]})
 
     async def updateUsername(self, username):
         if '@' in username: username.replace('@', '')
@@ -527,11 +586,38 @@ class Client(object):
 		}
         return await self.send('updateProfile', data)
 
-    async def createPoll(self):
-        pass
+    async def createPoll(self, object_guid, question='این نظر سنجی توسط rubpy ارسال شده است و برای تست است',
+        allows_multiple_answers=False, is_anonymous=True, reply_to_message_id=0,
+        type='Regular', options=['rubpy', 'rubpy'], correct_option_index=None, explanation=None):
+            if type == 'Regular':
+                data = {
+                    "allows_multiple_answers": allows_multiple_answers,
+                    "is_anonymous": is_anonymous,
+                    "object_guid": object_guid,
+                    "options": options,
+                    "question": question,
+                    "reply_to_message_id": int(reply_to_message_id),
+                    "rnd": randint(100000000, 999999999),
+                    "type": type
+                }
+            else:
+                data = {
+                    "allows_multiple_answers": allows_multiple_answers,
+                    "correct_option_index": correct_option_index,
+                    "is_anonymous": is_anonymous,
+                    "object_guid": object_guid,
+                    "options": options,
+                    "question": question,
+                    "reply_to_message_id": reply_to_message_id,
+                    "rnd": randint(100000000, 999999999),
+                    "type": "Quiz"
+                }
+                if explanation != None:
+                    data['explanation'] = explanation
+            return await self.send('createPoll', data)
 
-    async def votePoll(self):
-        pass
+    async def votePoll(self, poll_id, selection_index):
+        return await self.send('createPoll', {'poll_id': poll_id, 'selection_index': selection_index})
 
     async def getPollStatus(self, poll_id):
         return await self.send('getPollStatus', {'poll_id': poll_id}, 5)
@@ -540,7 +626,7 @@ class Client(object):
         data = {'poll_id': poll_id, 'selection_index': selection_index, 'start_id': start_id}
         return await self.send('getPollOptionVoters', data)
 
-    async def getLinkFromAppUrl(self, app_url: str) -> dict:
+    async def getLinkFromAppUrl(self, app_url):
         return await self.send('getLinkFromAppUrl', {'app_url': app_url})
 
     async def joinChannelAction(self, channel_guid, action='Join'):
@@ -567,6 +653,9 @@ class Client(object):
         data = {'channel_type': channel_type, 'title': channel_title, 'member_guids': member_guids or []}
         return await self.send('addChannel', data, 5)
 
+    async def removeChannel(self, channel_guid):
+        return await self.send('removeChannel', {'channel_guid': channel_guid})
+
     async def addFolder(self, name, exclude_chat_types=[], exclude_object_guids=[], include_chat_types=[], include_object_guids=[], is_add_to_top=True, folder_id=''):
         data = dict(exclude_object_guids=exclude_object_guids, include_object_guids=include_object_guids, exclude_chat_types=exclude_chat_types, include_chat_types=include_chat_types, folder_id=folder_id, is_add_to_top=is_add_to_top, name=name)
         return await self.send('addFolder', data, 5)
@@ -579,6 +668,14 @@ class Client(object):
         data = {'new_hint': new_hint, 'new_password': new_password, 'password': old_password}
         return await self.send('changePassword', data, 5)
 
+    async def requestChangePhoneNumber(self, new_phone_numer):
+        if new_phone_numer.startswith('0'):
+            phone_number = f'98{new_phone_numer[:1]}'
+        elif new_phone_numer.startswith('+98'):
+            phone_number = f'98{new_phone_numer[:1]}'
+        data = {'new_phone_number': phone_number}
+        return await self.send('requestChangePhoneNumber', data)
+
     async def turnOffTwoStep(self, password):
         return await self.send('turnOffTwoStep', {'password': password}, 5)
 
@@ -612,6 +709,9 @@ class Client(object):
         data = {'object_guid': object_guid, 'new_owner_user_guid': new_owner_user_guid}
         return await self.send('requestChangeObjectOwner', data)
 
+    async def getBotInfo(self, bot_guid):
+        return await self.send('getBotInfo', {'bot_guid': bot_guid}, 5)
+
     def Handler(self, func):
         async def runner():
             async for i in self.websocket():
